'''
Author: Dorad, ddxi@qq.com
Date: 2023-03-27 21:47:08 +02:00
LastEditors: Dorad, ddxi@qq.com
LastEditTime: 2023-03-28 04:29:53 +02:00
FilePath: \rrrcn\Base.py
Description: 

Copyright (c) 2023 by Dorad (ddxi@qq.com), All Rights Reserved.
'''

import os
import datetime
from abc import ABCMeta, abstractmethod

import requests
import pytz
from sqlalchemy import Table, Column, Integer, String, DateTime, MetaData, UniqueConstraint, Float, create_engine, inspect
from sqlalchemy.dialects.sqlite import insert
from sqlalchemy.orm import sessionmaker


cn_tz = pytz.timezone('Asia/Shanghai')


class BaseStation():
    __metaclass__ = ABCMeta

    def __init__(self):
        self.base_url = 'https://www.mwr.cn'
        self.action_url = ''
        self.session = requests.Session()
        self.session.headers.update({
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.114 Safari/537.36',
            'Accept': 'application/json, text/javascript, */*; q=0.01',
        })
        self.metadata = MetaData()
        self.Table = Table(
            'base_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String(20), comment='站码', nullable=False),
            Column('TM', DateTime, comment='采集时间', nullable=False),
            Column('CollectedAt', DateTime, comment='同步时间', nullable=False),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    @abstractmethod
    def query_data_at(self) -> list[dict]:
        return []

    @abstractmethod
    def query_history_data(self, start_at: datetime, end_at: datetime) -> list[dict]:
        """query history data from start_at to end_at

        Args:
            start_at (datetime): start time
            end_at (datetime): end time

        Returns:
            list[dict]: a list of dict, each dict is a data entry
        """
        return []

    @staticmethod
    def _strip_str_in_each_key(item_list):
        for item in item_list:
            for key in item:
                if isinstance(item[key], str):
                    item[key] = item[key].strip()
        return item_list

    def _parse_data_list(self, data_list: list[dict]) -> list[dict]:
        return data_list

    def save_data_to_db(self, data_list: list[dict], db_config_url: str = 'sqlite:///db/rrrcn.db'):
        if not os.path.exists(os.path.dirname(db_config_url[10:])):
            os.makedirs(os.path.dirname(db_config_url[10:]))
        engine = create_engine(db_config_url)
        Session = sessionmaker(bind=engine)
        session = Session()
        # create table if not exists
        if not inspect(engine).has_table(self.Table.name):
            self.Table.create(engine, checkfirst=True)
        # insert data without conflict
        if not data_list:
            return
        # split data_list into chunks
        chunk_size = 2000
        data_list_chunks = [data_list[i:i + chunk_size]
                            for i in range(0, len(data_list), chunk_size)]
        for data_list_chunk in data_list_chunks:
            session.execute(
                insert(self.Table).values(
                    data_list_chunk).on_conflict_do_nothing()
            )
        session.commit()
        session.close()
