
from datetime import datetime
import re
from .Base import BaseStation, cn_tz, Table, Column, Integer, String, DateTime, MetaData, UniqueConstraint, Float

base_url = 'http://113.57.190.228:8001'




class PopularStationReport(BaseStation):
    def __init__(self):
        super(PopularStationReport, self).__init__()
        self.url = base_url + '/Web/Report/GetRiverData'
        self.Table = Table(
            'hubei_popular_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String(20), comment='站码', nullable=False),
            Column('STNM', String(20), comment='站名', nullable=False),
            Column('RVNM', String(20), comment='河流'),
            Column('Z', Float(10, 4), comment='水位(米)', nullable=False),
            Column('WPTN', String(20), comment='水势'),
            # Column('Q', Float(10, 4), comment='流量(立方米/秒)', nullable=False),
            Column('YZ', Float(10, 4), comment='比昨日(+/-,涨/落)'),
            Column('FRZ', Float(10, 4), comment='设防水位'),
            Column('WRZ', Float(10, 4), comment='警戒水位'),
            Column('GRZ', Float(10, 4), comment='保证水位'),
            Column('MAXZ', Float(10, 4), comment='最高水位'),
            Column('INQ', Float(10, 4), comment='入流量(立方米/秒)'),
            Column('OTQ', Float(10, 4), comment='出流量(立方米/秒)'),
            Column('TM', DateTime, comment='采集时间', nullable=False),
            Column('CollectedAt', DateTime, comment='采集时间',
                   nullable=False, default=datetime.now(tz=cn_tz)),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def query_latest_data(self) -> list[dict]:
        """query latest data of current hour

        Returns:
            list[dict]: a list of dict, each dict is a data entry
        """
        return self.query_data_at(datetime.now(cn_tz))

    def query_data_at(self, dateTime:datetime) -> list[dict]:
        """query data at specific time

        Args:
            dateTime (datetime): the datetime to query

        Raises:
            Exception: if the status code is not 200

        Returns:
            list[dict]: a list of dict, each dict is a data entry
        """        
        dateTime = dateTime.astimezone(cn_tz).replace(minute=0, second=0, microsecond=0)
        response = self.session.get(
            self.url+'?date=%s:00' % (dateTime.strftime('%Y-%m-%d+%H')))
        if response.status_code != 200:
            raise Exception(
                'Failed to query latest data, status code: %s' % response.status_code)
        return self._parse_data_list(response.json()['rows'], dateTime)

    def _parse_data_list(self, data_list: list[dict], dateTime: datetime) -> list[dict]:
        def __parse_Q(str):
            inout = [None, None]
            if str.find('入') > -1:
                inout[0] = re.findall('([\d.]+)\(入\)', str)[0]
            if str.find('出') > -1:
                inout[1] = re.findall('([\d.]+)\(出\)', str)[0]
            if str != '' and str.find('出') < 0 and str.find('入') < 0:
                inout[1] = float(str)
            return inout

        def _parse_wptn(wptn: str):
            key_map = {
                '4': '降',
                '5': '升',
                '6': '平',
            }
            return key_map.get(wptn, '缺测')
        ndata_list = []
        for row in data_list:
            for r in [{
                key: row[key] for key in filter(lambda x: '1' not in x, row.keys())
            }, {
                key.replace('1', ''): row[key] for key in filter(lambda x: '1' in x, row.keys())
            }]:
                if not r['Z']:
                    continue
                r.update({
                    "WPTN": _parse_wptn(r['WPTN']),
                    "Z": None if r['Z'] == '' else float(r['Z']),
                    "YZ": None if r['YZ'] == '' else float(r['YZ']),
                    "INQ": __parse_Q(r['Q'])[0],
                    "OTQ": __parse_Q(r['Q'])[1],
                    "FRZ": None if r['FRZ'] == '--' else float(r['FRZ']),
                    "WRZ": None if r['WRZ'] == '--' else float(r['WRZ']),
                    "GRZ": None if r['GRZ'] == '--' else float(r['GRZ']),
                    "MAXZ": None if r['MAXZ'] == '--' else float(r['MAXZ']),
                    "TM": dateTime,
                })
                r.pop('Q')
                ndata_list.append(r)
        return ndata_list


class RiverStationReport(PopularStationReport):
    def __init__(self):
        super(RiverStationReport, self).__init__()
        self.url = base_url + '/Web/Report/GetRiverReportData'
        self.Table = Table(
            'hubei_river_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String, nullable=False,
                   comment='站码', index=True),
            Column('STNM', String, nullable=False, comment='站名'),
            Column('STTP', String, comment='站类'),
            Column('ADDVNM', String, comment='市（州）'),
            Column('Z', Float(10, 4), nullable=False, comment='水位(米)'),
            Column('WPTN', String, comment='水势'),
            Column('Q', Float(10, 4), comment='流量(立方米/秒)'),
            Column('BJZ', Float(10, 4), comment='比昨日(+/-,涨/落)'),
            Column('TM', DateTime, nullable=False, comment='采集时间'),
            Column('CollectedAt', DateTime, default=datetime.now(cn_tz),
                   nullable=False, comment='同步时间'),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def _parse_data_list(self, data_list: list[dict], dateTime: datetime) -> list[dict]:
        def _parse_Z(s):
            if not s:
                return None
            if len(re.findall('([\d.]+)', s)):
                return float(re.findall('([\d.]+)', s)[0])
            return None

        def _parse_wptn(wptn: str):
            key_map = {
                '4': '降',
                '5': '升',
                '6': '平',
            }
            return key_map.get(wptn, '缺测')
        ndata_list = []
        for row in data_list:
            for r in [{
                key: row[key] for key in row.keys() if '1' not in key
            }, {
                key.replace('1', ''): row[key] for key in row.keys() if '1' in key
            }]:
                if not r['Z']:
                    continue
                r.update({
                    'Z': _parse_Z(r['Z']),
                    'WPTN': _parse_wptn(r['WPTN']),
                    'Q': None if r['Q'] == '' else float(r['Q']),
                    'BJZ': 0 if r['BJZ'] == '--' else float(r['BJZ']),
                    'TM': dateTime,
                })
                ndata_list.append(r)
        return ndata_list


class ReservoirStationReport(PopularStationReport):
    def __init__(self):
        super(ReservoirStationReport, self).__init__()
        self.url = base_url + '/Web/Report/GetBigMSKReport'
        self.Table = Table(
            'hubei_reservoir_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String, nullable=False, comment='站码', index=True),
            Column('STNM', String, nullable=False, comment='站名'),
            Column('ADDVNM', String, comment='市（州）'),
            Column('COUNTY', String, comment='所在县'),
            Column('DANGER', String, comment='是否病险'),
            Column('AREA', Float(10, 4), comment='承雨面积'),
            Column('SPILLWAY', String, comment='溢洪道(泄洪洞)闸门'),
            Column('SPILLWAYALTITUDE', String, comment='溢洪道(泄洪洞)底板高程'),
            Column('DAMEL', Float(10, 4), comment='坝顶高程'),
            Column('FSLTDZ', Float(10, 4),  comment='汛限水位'),
            Column('NORMZ', Float(10, 4),  comment='正常蓄水位'),
            Column('FHGSW', Float(10, 4),  comment='防洪高水位'),
            Column('DSFLZ', Float(10, 4),  comment='设计水位'),
            Column('CKFLZ', Float(10, 4),  comment='校核水位'),
            Column('RZ', Float(10, 4), comment='库水位'),
            Column('XXB', Float(10, 4), comment='比汛限'),
            Column('W', Float(10, 4), comment='蓄水量'),
            Column('YXW', String, comment='YXW'),
            Column('INQ', Float(10, 4), comment='入库流量(立方米/秒)'),
            Column('OTQ', Float(10, 4), comment='出库流量(立方米/秒)'),
            Column('BJYMONTH', Float(10, 4), comment='BJYMONTH'),
            Column('BJMYEAR', Float(10, 4), comment='BJMYEAR'),
            Column('BJYMONTHW', Float(10, 4), comment='BJYMONTHW'),
            Column('BJMYEARW', Float(10, 4), comment='BJMYEARW'),
            Column('EIGHT', Float(10, 4), comment='与最近一次8时比'),
            Column('ONE', Float(10, 4), comment='与1小时前比'),
            Column('TWENTYFOUR', Float(10, 4), comment='与24小时前比'),
            Column('GATESTATE', String, comment='溢洪道(泄洪洞)开闸情况'),
            Column('OTHERGATESTATE', String, comment='输水管及其他放水设施开闸情况'),
            Column('OUT', String, comment='OUT'),
            Column('TM', DateTime, comment='采集时间'),
            Column('CollectedAt', DateTime, comment='同步时间',
                   default=datetime.now(cn_tz)),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def _parse_data_list(self, data_list: list[dict], dateTime: datetime) -> list[dict]:
        for row in data_list:
            for key in ['AREA', 'DAMEL', 'FSLTDZ', 'NORMZ', 'FHGSW', 'DSFLZ', 'CKFLZ', 'RZ', 'XXB', 'W', 'INQ', 'OTQ', 'BJYMONTH', 'BJMYEAR', 'BJYMONTHW', 'BJMYEARW', 'EIGHT', 'ONE', 'TWENTYFOUR']:
                row[key] = None if row[key] in [
                    '--', '', None, '无'] else float(row[key])
            row['TM'] = dateTime
        return data_list


class RainfallStation(BaseStation):
    def __init__(self):
        super(RainfallStation, self).__init__()
        self.url = base_url + '/Web/WRSearch/GetRainSearchData'
        self.Table = Table(
            'hubei_rainfall_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String, nullable=False, comment='站码', index=True),
            Column('STNM', String, nullable=False, comment='站名'),
            Column('TM', DateTime, comment='采集时间'),
            Column('DRP', Float(10, 4), comment='降雨量(mm)'),
            Column('INTV', Integer, comment='时段长'),
            Column('PDR', Integer, comment='降雨历时'),
            Column('DYP', Float(10, 4), comment='日雨量(mm)'),
            Column('WTH', String, comment='天气'),
            Column('CollectedAt', DateTime, comment='同步时间',
                   default=datetime.now(cn_tz)),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )
    def query_latest_data(self) -> list[dict]:
        """query latest data of current hour

        Returns:
            list[dict]: a list of data, each dict is a data entry
        """        
        return self.query_data_at(datetime.now(cn_tz))

    def query_history_data(self, start_at: datetime, end_at: datetime) -> list[dict]:
        """query history data between start_at and end_at

        Args:
            start_at (datetime): the start time of query
            end_at (datetime): the end time of query

        Raises:
            Exception: if the start_at is greater than end_at
            Exception: if the status code of response is not 200

        Returns:
            list[dict]: a list of data, each dict is a data entry
        """        
        start_at = start_at.astimezone(cn_tz).replace(minute=0, second=0, microsecond=0)
        end_at = end_at.astimezone(cn_tz).replace(minute=0, second=0, microsecond=0)
        if start_at > end_at:
            raise Exception('start_at must be less than end_at')
        # featch data
        response = self.session.get(self.url+'?type=0&addvcd=0&stcd=0&sdate=%s:00&edate=%s:00' % (start_at.strftime('%Y-%m-%d %H'), end_at.strftime('%Y-%m-%d %H')))
        if response.status_code != 200:
            raise Exception(
                'Failed to query latest data, status code: %s' % response.status_code)
        return self._parse_data_list(response.json()['rows'])

    def query_data_at(self, dateTime:datetime) -> list[dict]:
        """ query data at a specific time

        Args:
            dateTime (datetime): the time of query

        Returns:
            list[dict]: a list of data, each dict is a data entry
        """
        dateTime = dateTime.replace(minute=0, second=0, microsecond=0)
        return self.query_history_data(start_at=dateTime, end_at=dateTime)

    def _parse_data_list(self, data_list: list[dict]) -> list[dict]:
        for row in data_list:
            for key in ['DRP', 'DYP']:
                row[key] = None if row[key] in [
                    '--', '', None, '无'] else float(row[key])
            for key in ['INTV', 'PDR']:
                row[key] = None if row[key] in [
                    '--', '', None, '无'] else int(row[key])
            row['TM'] = datetime.strptime(row['TM'], '%Y-%m-%d %H:%M')
        return data_list

class WeirGateStation(RainfallStation):
    def __init__(self):
        super(WeirGateStation, self).__init__()
        self.url = base_url + '/Web/WRSearch/GetYZSearchData'
        self.Table = Table(
            'hubei_weir_gate_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String, nullable=False, comment='站码', index=True),
            Column('STNM', String, nullable=False, comment='站名'),
            Column('TM', DateTime, comment='采集时间'),
            Column('UPZ', Float(10, 4), comment='闸上水位'),
            Column('SUPWPTN', String, comment='闸上水势'),
            Column('DWZ', Float(10, 4), comment='闸下水位'),
            Column('SDWWPTN', String, comment='闸下水势'),
            Column('TGTQ', Float(10, 4), comment='总过闸流量'),
            Column('SWCHRCD', String, comment='闸水特征码'),
            Column('CollectedAt', DateTime, comment='同步时间',default=datetime.now(cn_tz)),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def _parse_data_list(self, data_list: list[dict]) -> list[dict]:
        def _parse_wptn(wptn: str):
            key_map = {
                '4': '降',
                '5': '升',
                '6': '平',
            }
            return key_map.get(wptn, '缺测')
        for r in data_list:
            for key in ['UPZ', 'DWZ', 'TGTQ']:
                r[key] = None if r[key] in [
                    '--', '', None, '无'] else float(r[key])
            for key in ['SUPWPTN', 'SDWWPTN']:
                r[key] = _parse_wptn(r[key])
            r['TM'] = datetime.strptime(r['TM'], '%Y-%m-%d %H:%M')
        return data_list
