'''
Author: Dorad, ddxi@qq.com
Date: 2023-03-27 21:46:45 +02:00
LastEditors: Dorad, ddxi@qq.com
LastEditTime: 2023-03-28 04:07:46 +02:00
FilePath: \rrrcn\MWR.py
Description: 

Copyright (c) 2023 by Dorad (ddxi@qq.com), All Rights Reserved.
'''

from .Base import BaseStation, cn_tz, Table, Column, Integer, String, DateTime, Float, MetaData, UniqueConstraint
from datetime import datetime

base_url = 'http://xxfb.mwr.cn'


class RainfallStation(BaseStation):
    def __init__(self):
        super().__init__()
        self.url = base_url+'/hydroSearch/pointHydroInfo'
        self.Table = Table(
            'mwr_rainfall_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String(20), comment='站码', nullable=False),
            Column('STNM', String(50), comment='站名', nullable=False),
            Column('POIADDV', String(20), comment='行政区'),
            Column('POIBSNM', String(30), comment='流域'),
            Column('RVNM', String(20), comment='河名'),
            Column('WEBSTLC', String(50), comment='站址'),
            Column('LAT', Float, comment='纬度'),
            Column('LGT', Float, comment='经度'),
            Column('TM', DateTime, comment='采样时间', nullable=False),
            Column('DYP', Float, comment='日降雨量(毫米)', nullable=False),
            Column('WTH', String(20), comment='天气'),
            Column('CollectedAt', DateTime, comment='同步时间',
                   default=datetime.now(cn_tz), nullable=False),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def query_latest_data(self) -> list[dict]:
        response = self.session.post(self.url, data={})
        if response.status_code != 200:
            raise Exception(
                'Failed to query latest data, status code: %s' % response.status_code)
        return self._parse_data_list(self._strip_str_in_each_key(response.json()['result']['data']))

    def _parse_data_list(self, data_list: list[dict]) -> list[dict]:
        for row in data_list:
            # rename the key to upper case
            for key in list(row.keys()):
                row[key.upper()] = row.pop(key)
            # convert the time to datetime
            row.pop('DATETIME')
            row['TM'] = datetime.strptime(row['TM'], '%Y-%m-%d %H:%M:%S')
            for key in ['DYP', 'LAT', 'LGT']:
                row[key] = float(row[key])
        return data_list


class RiverStation(RainfallStation):
    def __init__(self):
        super(RainfallStation, self).__init__()
        self.url = base_url+'/hydroSearch/greatRiver'
        self.Table = Table(
            'mwr_river_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String(20), comment='站码', nullable=False),
            Column('STNM', String(50), comment='站名', nullable=False),
            Column('POIADDV', String(20), comment='行政区'),
            Column('POIBSNM', String(30), comment='流域'),
            Column('RVNM', String(20), comment='河名'),
            Column('WEBSTLC', String(50), comment='站址'),
            Column('WRZ', Float, comment='警戒水位(米)'),
            Column('TM', String(20), comment='时间', nullable=False),
            Column('ZL', Float, comment='水位(米)'),
            Column('QL', Float, comment='流量(立方米/秒)'),
            Column('CollectedAt', DateTime, comment='同步时间',
                   default=datetime.now(cn_tz), nullable=False),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def _parse_data_list(self, data_list: list[dict]) -> list[dict]:
        for row in data_list:
            # rename the key to upper case
            for key in list(row.keys()):
                row[key.upper()] = row.pop(key)
            # convert the time to datetime
            row.pop('DATETIME')
            for key in ['WRZ', 'ZL', 'QL']:
                row[key] = float(row[key])
            row['TM'] = datetime.strptime(row['TM'], '%Y-%m-%d %H:%M:%S')
        return data_list


class ReservoirStation(RainfallStation):
    def __init__(self):
        super(ReservoirStation, self).__init__()
        self.url = base_url+'/hydroSearch/greatRsvr'
        self.Table = Table(
            'mwr_reservoir_stations',
            self.metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('STCD', String(20), comment='站码', nullable=False),
            Column('STNM', String(50), comment='站名', nullable=False),
            Column('DAMEL', Float, comment='坝高(米)'),
            Column('POIADDV', String(20), comment='行政区'),
            Column('POIBSNM', String(30), comment='流域'),
            Column('RVNM', String(20), comment='河名'),
            Column('WEBSTLC', String(50), comment='站址'),
            Column('TM', String(20), comment='时间', nullable=False),
            Column('WL', Float, comment='蓄水量(百万立方米)'),
            Column('INQ', Float, comment='入库流量(立方米/秒)'),
            Column('RZ', Float, comment='库水位(米)'),
            Column('CollectedAt', DateTime, comment='同步时间',
                   default=datetime.now(cn_tz), nullable=False),
            UniqueConstraint('STCD', 'TM', name='uix_STCD_TM'),
        )

    def _parse_data_list(self, data_list: list[dict]) -> list[dict]:
        for row in data_list:
            # rename the key to upper case
            for key in list(row.keys()):
                row[key.upper()] = row.pop(key)
            # convert the time to datetime
            row.pop('DATETIME')
            for key in ['DAMEL', 'WL', 'INQ', 'RZ']:
                row[key] = float(row[key])
            row['TM'] = datetime.strptime(row['TM'], '%Y-%m-%d %H:%M:%S')
        return data_list
